import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from pyearth.system.define_global_variables import *
from pyearth.visual.color.create_diverge_rgb_color_hex import create_diverge_rgb_color_hex

def barplot_data(aData_in,
                 aLabel_x_in,
                 aLabel_y_in,
                 sFilename_out,
                 iDPI_in=None,
                 ncolumn_in=None,
                 iSize_x_in=None,
                 iSize_y_in=None,
                 dMax_y_in=None,
                 dMin_y_in=None,
                 dSpace_y_in=None,
                 aMarker_in=None,
                 aColor_in=None,
                 aHatch_in=None,
                 sLabel_y_in=None,
                 aLabel_legend_in=None,
                 aLocation_legend_in=None,
                 sFormat_y_in=None,
                 sLocation_legend_in=None,
                 sTitle_in=None):
    """_summary_

    Args:
        aData_in (_type_): _description_
        aLabel_x_in (_type_): _description_
        aLabel_y_in (_type_): _description_
        sFilename_out (_type_): _description_
        iDPI_in (_type_, optional): _description_. Defaults to None.
        ncolumn_in (_type_, optional): _description_. Defaults to None.
        iSize_x_in (_type_, optional): _description_. Defaults to None.
        iSize_y_in (_type_, optional): _description_. Defaults to None.
        dMax_y_in (_type_, optional): _description_. Defaults to None.
        dMin_y_in (_type_, optional): _description_. Defaults to None.
        dSpace_y_in (_type_, optional): _description_. Defaults to None.
        aMarker_in (_type_, optional): _description_. Defaults to None.
        aColor_in (_type_, optional): _description_. Defaults to None.
        aHatch_in (_type_, optional): _description_. Defaults to None.
        sLabel_y_in (_type_, optional): _description_. Defaults to None.
        aLabel_legend_in (_type_, optional): _description_. Defaults to None.
        aLocation_legend_in (_type_, optional): _description_. Defaults to None.
        sFormat_y_in (_type_, optional): _description_. Defaults to None.
        sLocation_legend_in (_type_, optional): _description_. Defaults to None.
        sTitle_in (_type_, optional): _description_. Defaults to None.
    """

    aData_in = np.array(aData_in)
    pShape = aData_in.shape

    nData = pShape[0]  # len(aLabel_x_in) this maybe checked

    if iDPI_in is not None:
        iDPI = iDPI_in
    else:
        iDPI = 300

    if iSize_x_in is not None:
        iSize_x = iSize_x_in
    else:
        iSize_x = 12

    if iSize_y_in is not None:
        iSize_y = iSize_y_in
    else:
        iSize_y = 9

    if dMax_y_in is not None:
        dMax_y = dMax_y_in
    else:
        dMax_y = np.nanmax(aData_in) * 1.0

    if dMin_y_in is not None:
        dMin_y = dMin_y_in
    else:
        dMin_y = np.nanmin(aData_in)

    if (dMax_y <= dMin_y):
        return

    if sLabel_y_in is not None:
        sLabel_y = sLabel_y_in
    else:
        sLabel_y = ''

    if sTitle_in is not None:
        sTitle = sTitle_in
    else:
        sTitle = ''

    if sFormat_y_in is not None:
        iFlag_format_y = 1
        sFormat_y = sFormat_y_in
    else:
        iFlag_format_y = 0

    if sLocation_legend_in is not None:
        sLocation_legend = sLocation_legend_in
    else:
        sLocation_legend = "upper right"

    if aLocation_legend_in is not None:
        aLocation_legend = aLocation_legend_in
    else:
        aLocation_legend = (1.0, 1.0)

    if ncolumn_in is not None:
        ncolumn = ncolumn_in
    else:
        ncolumn = 1

    if aColor_in is not None:
        aColor = aColor_in
    else:
        if (nData >= 3):
            aColor = create_diverge_rgb_color_hex(nData)
        else:
            if nData == 2:
                aColor = ['red', 'blue']
            else:
                aColor = ['red']

    if aHatch_in is not None:
        aHatch = aHatch_in
    else:
        aHatch = np.fill(nData, '+')

    fig = plt.figure(dpi=iDPI)
    fig.set_figwidth(iSize_x)
    fig.set_figheight(iSize_y)
    ax = fig.add_axes([0.1, 0.5, 0.8, 0.4])
    x = np.arange(len(aLabel_x_in))
    # Add some text for labels, title and custom x-axis tick labels, etc.
    ax.set_ylabel(sLabel_y)
    ax.set_title(sTitle)
    ax.set_xticks(x)
    ax.set_xticklabels(aLabel_x_in)

    total_width = 0.7
    width = total_width / nData

    for i in np.arange(1, nData + 1, 1):

        data1 = aData_in[i-1]
        # need bettern control here, hatch (texture) can be used to betten represet some data
        if i == 1 or i == 2:
            rects = ax.bar(x - total_width * 0.5 + i * width, data1, width, label=aLabel_y_in[i-1],
                           color=aColor[i-1], hatch=aHatch[i-1], edgecolor="k")
        else:
            rects = ax.bar(x - total_width * 0.5 + i * width, data1, width, label=aLabel_y_in[i-1],
                           color=aColor[i-1])  # , hatch = aHatch[i-1], edgecolor = "k")

        pass

    if (iFlag_format_y == 1):
        ax.yaxis.set_major_formatter(mpl.ticker.FormatStrFormatter(sFormat_y))

    ax.set_ylim(dMin_y, dMax_y)
    ax.legend(bbox_to_anchor=aLocation_legend,
              loc=sLocation_legend,
              fontsize=12,
              ncol=ncolumn)
    plt.savefig(sFilename_out, bbox_inches='tight')

    plt.close('all')
    plt.clf()

    return



import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from pyearth.system.define_global_variables import *
from pyearth.visual.color.create_diverge_rgb_color_hex import create_diverge_rgb_color_hex


def barplot_data_with_reference(aData_in,
                                aLabel_x_in,
                                aLabel_y_in,
                                sFilename_out,
                                aLabel_z_in=None,
                                aData_reference_in=None,
                                aLabel_legend_reference_in=None,
                                iDPI_in=None,
                                iFlag_scientific_notation_in=None,
                                ncolumn_in=None,
                                iSize_x_in=None,
                                iSize_y_in=None,
                                dMax_y_in=None,
                                dMin_y_in=None,
                                dSpace_y_in=None,
                                aMarker_in=None,
                                aColor_in=None,
                                aHatch_in=None,
                                sLabel_info_in=None,
                                sLabel_y_in=None,
                                aLinestyle_in=None,
                                aLocation_legend_in=None,
                                sFormat_y_in=None,
                                sLocation_legend_in=None,
                                sTitle_in=None):

    aData_in = np.array(aData_in)
    aData_reference_in = np.array(aData_reference_in)
    pShape = aData_in.shape
    ndim = aData_in.ndim
    if ndim == 2:
        iFlag_sub = 0
        nCat = pShape[0]
        nData = pShape[1]

    else:
        iFlag_sub = 1
        nCat = pShape[0]
        nsub = pShape[1]
        nData = pShape[2]

    if aData_reference_in is not None:
        iFlag_ref = 1
        nData_reference = aData_reference_in.shape[0]

    else:
        iFlag_ref = 0

    if iFlag_scientific_notation_in is not None:
        iFlag_scientific_notation = 1
    else:
        iFlag_scientific_notation = 0

    if iDPI_in is not None:
        iDPI = iDPI_in
    else:
        iDPI = 300

    if iSize_x_in is not None:
        iSize_x = iSize_x_in
    else:
        iSize_x = 12

    if iSize_y_in is not None:
        iSize_y = iSize_y_in
    else:
        iSize_y = 9

    if dMax_y_in is not None:
        dMax_y = dMax_y_in
    else:
        dMax_y = np.nanmax(aData_in) * 1.0

    if dMin_y_in is not None:
        dMin_y = dMin_y_in
    else:
        dMin_y = np.nanmin(aData_in)  # if it has negative value, change here

    if (dMax_y <= dMin_y):
        return

    if sLabel_y_in is not None:
        sLabel_y = sLabel_y_in
    else:
        sLabel_y = ''

    if sTitle_in is not None:
        sTitle = sTitle_in
    else:
        sTitle = ''

    if sFormat_y_in is not None:
        iFlag_format_y = 1
        sFormat_y = sFormat_y_in
    else:
        iFlag_format_y = 0

    if sLocation_legend_in is not None:
        sLocation_legend = sLocation_legend_in
    else:
        sLocation_legend = "upper right"

    if aLocation_legend_in is not None:
        aLocation_legend = aLocation_legend_in
    else:
        aLocation_legend = (1.0, 1.0)

    if ncolumn_in is not None:
        ncolumn = ncolumn_in
    else:
        ncolumn = 1

    if aColor_in is not None:
        aColor = aColor_in
    else:
        if (nData >= 3):
            aColor = create_diverge_rgb_color_hex(nData+nData_reference)
        else:
            if nData == 2:
                aColor = ['red', 'blue']
            else:
                aColor = ['red']

    if aHatch_in is not None:
        aHatch = aHatch_in
    else:
        aHatch = np.fill(nData, '+')

    fig = plt.figure(dpi=iDPI)
    fig.set_figwidth(iSize_x)
    fig.set_figheight(iSize_y)
    ax = fig.add_axes([0.1, 0.5, 0.8, 0.4])
    x = np.arange(len(aLabel_x_in))
    dMin_x = -0.5
    dMax_x = len(aLabel_x_in)-0.5

    ax.set_ylabel(sLabel_y, fontsize=14)
    ax.set_title(sTitle, fontsize=14)
    ax.set_xticks(x)
    ax.set_xticklabels(aLabel_x_in)
    ax.tick_params(axis="x", labelsize=15)
    ax.tick_params(axis="y", labelsize=15)

    total_width = 0.6

    aLegend_artist = []
    aLabel = []
    if ndim == 2:
        width = total_width / (nData)
        for i in range(0, nCat, 1):
            for j in np.arange(0, nData, 1):
                data1 = aData_in[j, i]
                x1 = x[i] - total_width * 0.5 + (j+0.5) * width
                rects = ax.bar(x1, data1, width, label=aLabel_y_in[k], linestyle=aLinestyle_in[k],
                               color=aColor[k], hatch=aHatch[k], edgecolor="k")

                pass
        if iFlag_ref == 1:
            for i in aData_reference_in:
                x0 = [-1, nData-len(aData_reference_in)]
                y0 = [aData_in[i][0], aData_in[i][0]]
                ax.plot(x0, y0,
                        color=aColor[i], linestyle='dashed',
                        marker=aMarker_in[i],
                        label=aLabel_y_in[i])
    else:
        width = total_width / (nData * nsub)
        if iFlag_ref == 1:
            for i in range(nData_reference):
                x0 = [-1, nCat]
                y0 = [aData_reference_in[i], aData_reference_in[i]]
                line, = ax.plot(x0, y0,
                                color=aColor[i+nData], linestyle='dashed',
                                marker=aMarker_in[i],
                                label=aLabel_legend_reference_in[i])
                aLegend_artist.append(line)
                aLabel.append(aLabel_legend_reference_in[i])

        for i in range(0, nCat, 1):
            for j in np.arange(0, nsub, 1):
                for k in np.arange(0, nData, 1):
                    data1 = aData_in[i, j, k]
                    x1 = x[i] - total_width * 0.5 + j*width + k * (width * 2)
                    # print(x1)
                    if j == 0 and i == 0:
                        rects, = ax.bar(x1, data1, width, label=aLabel_y_in[k], linestyle=aLinestyle_in[k],
                                        color=aColor[k], hatch=aHatch[j], edgecolor="k")
                        aLegend_artist.append(rects)
                        aLabel.append(aLabel_y_in[k])

                    else:
                        rects, = ax.bar(x1, data1, width,  linestyle=aLinestyle_in[k],
                                        color=aColor[k], hatch=aHatch[j], edgecolor="k")

        pass

    if iFlag_scientific_notation == 1:
        formatter = mpl.ticker.ScalarFormatter(useMathText=True)
        formatter.set_scientific(True)
        # formatter.set_powerlimits((-1,1)) # you might need to change here
        ax.yaxis.set_major_formatter(formatter)
        # most time, when you use scientific notation, you may not need set the space,
        # but you may still set it using the method below
        pass

    if sLabel_info_in is not None:
        ax.text(0.1, 0.9, sLabel_info_in,
                verticalalignment='center', horizontalalignment='left',
                transform=ax.transAxes,
                color='black', fontsize=13)

    if (iFlag_format_y == 1):
        ax.yaxis.set_major_formatter(mpl.ticker.FormatStrFormatter(sFormat_y))

    ax.set_xlim(dMin_x, dMax_x)
    ax.set_ylim(dMin_y, dMax_y)
    ax.grid(linewidth=1, color='gray', alpha=0.3, linestyle='--')

    if iFlag_sub == 1:
        handles = list()
        # labels = []
        for i in range(nsub):
            # handles.append(
            p = mpl.patches.Rectangle(
                (0, 0), 2, 2, hatch=aHatch[i], facecolor='w', label=aLabel_z_in[i])  # )
            aLegend_artist.append(p)
            aLabel.append(aLabel_z_in[i])

    ax.legend(aLegend_artist, aLabel,      bbox_to_anchor=aLocation_legend,
              loc=sLocation_legend,
              fontsize=14,
              ncol=ncolumn)

    plt.savefig(sFilename_out, bbox_inches='tight')

    plt.close('all')
    plt.clf()

    return



import numpy as np

import matplotlib as mpl
import matplotlib.pyplot as plt

from pyearth.system.define_global_variables import *
from pyearth.visual.color.create_diverge_rgb_color_hex import create_diverge_rgb_color_hex

def polygon_under_graph(xlist, ylist, z_level):
    """
    Construct the vertex list which defines the polygon filling the space under
    the (xlist, ylist) line graph.  Assumes the xs are in ascending order.
    """
    return [(xlist[0], z_level), *zip(xlist, ylist), (xlist[-1], z_level)]


def plot3d_time_series_data_fill(aTime_all,
                                 aData_all,
                                 sFilename_out,
                                 iDPI_in=None,
                                 iFlag_trend_in=None,
                                 iReverse_z_in=None,
                                 iSize_x_in=None,
                                 iSize_y_in=None,
                                 dMin_x_in=None,
                                 dMax_x_in=None,
                                 dMax_z_in=None,
                                 dMin_z_in=None,
                                 dSpace_x_in=None,
                                 dSpace_z_in=None,
                                 aColor_in=None,
                                 aLabel_y_in=None,
                                 sMarker_in=None,
                                 sLabel_x_in=None,
                                 sLabel_y_in=None,
                                 sLabel_z_in=None,
                                 sLabel_legend_in=None,
                                 sTitle_in=None):

    if iDPI_in is not None:
        iDPI = iDPI_in
    else:
        iDPI = 300

    if iFlag_trend_in is not None:
        iFlag_trend = 1
    else:
        iFlag_trend = 0

    if iReverse_z_in is not None:
        iReverse_z = 1
    else:
        iReverse_z = 0

    if iSize_x_in is not None:
        iSize_x = iSize_x_in
    else:
        iSize_x = 12

    if iSize_y_in is not None:
        iSize_y = iSize_y_in
    else:
        iSize_y = 9

    if dSpace_x_in is not None:
        dSpace_x = dSpace_x_in
    else:
        dSpace_x = 1

    if dSpace_z_in is not None:
        dSpace_z = dSpace_z_in
    else:
        dSpace_z = 1

    if sLabel_x_in is not None:
        sLabel_x = sLabel_x_in
    else:
        sLabel_x = ''

    if sLabel_y_in is not None:
        sLabel_y = sLabel_y_in
    else:
        sLabel_y = ''

    if sLabel_z_in is not None:
        sLabel_z = sLabel_z_in
    else:
        sLabel_z = ''

    if sLabel_legend_in is not None:
        sLabel_legend = sLabel_legend_in
    else:
        sLabel_legend = ''

    if sTitle_in is not None:
        sTitle = sTitle_in
    else:
        sTitle = ''

    if sMarker_in is not None:
        sMarker = sMarker_in
    else:
        sMarker = '+'

    # nstress = len(aTime)

    nslice = len(aData_all)
    if aColor_in is not None:
        aColor = aColor_in
    else:
        aColor = create_diverge_rgb_color_hex(nslice)

    if dMax_z_in is not None:
        dMax_z = dMax_z_in
    else:
        dMax_z = np.nanmax(aData_all)

    if dMin_z_in is not None:
        dMin_z = dMin_z_in
    else:
        dMin_z = np.nanmin(aData_all)  # if it has negative value, change here

    if (dMax_z <= dMin_z):
        return

    fig = plt.figure(dpi=iDPI)
    fig.set_figwidth(iSize_x)
    fig.set_figheight(iSize_y)

    ax = fig.add_subplot(111, projection='3d')
    # ax.pbaspect = np.array([3.0, 1.0, 1.0])
    # ax.set_box_aspect((np.ptp(xs), np.ptp(ys), np.ptp(zs)))

    verts = []
    ys = range(nslice)
    z_level = dMin_z
    for iSlice in np.arange(1, nslice+1, 1):
        xs = mpl.dates.date2num(aTime_all[iSlice-1])
        aData = aData_all[iSlice-1]
        nan_index = np.where(aData == missing_value)
        aData[nan_index] = np.nan
        good_index = np.where(~np.isnan(aData))
        # y1 = (aData[1])[0]
        # y_top = aData
        zs = aData
        verts.append(polygon_under_graph(xs, zs, z_level))
        pass

    poly = mpl.collections(verts, facecolors=aColor, alpha=.6)
    ax.add_collection3d(poly, zs=ys, zdir='y')
    pYear = mpl.dates.YearLocator(1)   # every year
    # pMonth = mpl.dates.MonthLocator()  # every month
    ax.axis('on')

    ax.xaxis._axinfo["grid"]['linewidth'] = 0.
    ax.yaxis._axinfo["grid"]['linewidth'] = 0.
    ax.zaxis._axinfo["grid"]['color'] = "grey"
    ax.zaxis._axinfo["grid"]['linestyle'] = "--"

    ax.xaxis.set_major_locator(pYear)
    # ax.xaxis.set_minor_locator(pMonth)
    sYear_format = mpl.dates.DateFormatter('%Y')

    ax.xaxis.set_major_formatter(sYear_format)
    ax.set_xlabel(sLabel_x)
    ax.xaxis.set_tick_params(labelsize=6)

    ax.set_xlim3d(np.min(aTime_all[0]), np.max(aTime_all[0]))
    # ax.xaxis.set_major_locator(mpl.ticker.MultipleLocator(dSpace_x))

    ax.set_ylim3d(nslice-1, 0)
    ax.set_yticks(range(nslice))
    aLabel_y = []
    for i in aLabel_y_in:
        aLabel_y.append(i.title())
        pass

    ax.set_yticklabels(aLabel_y, fontsize=8)

    ax.set_ylabel(sLabel_y)

    # ax.zaxis.set_major_locator(mpl.ticker.MultipleLocator(dSpace_z))
    if (iReverse_z == 1):
        ax.set_zlim3d(dMax_z, dMin_z)
    else:
        ax.set_zlim3d(dMin_z, dMax_z)
        pass

    ax.set_zlabel(sLabel_z)
    ax.set_box_aspect((6, 6, 3))

    plt.savefig(sFilename_out, bbox_inches='tight')

    plt.close('all')
    plt.clf()
    # print('finished plotting')



import numpy as np
from osgeo import gdalconst

def gdal_to_numpy_datatype(gdal_dtype):
    if gdal_dtype == gdalconst.GDT_Byte:
        return np.uint8
    elif gdal_dtype == gdalconst.GDT_UInt16:
        return np.uint16
    elif gdal_dtype == gdalconst.GDT_Int16:
        return np.int16
    elif gdal_dtype == gdalconst.GDT_UInt32:
        return np.uint32
    elif gdal_dtype == gdalconst.GDT_Int32:
        return np.int32
    elif gdal_dtype == gdalconst.GDT_Float32:
        return np.float32
    elif gdal_dtype == gdalconst.GDT_Float64:
        return np.float64
    elif gdal_dtype == gdalconst.GDT_Int8:
        return np.int8
    else:
        raise ValueError(f"GDAL data type {gdal_dtype} not recognized")

# Usage:
#numpy_dtype = gdal_to_numpy(gdalconst.GDT_Int16)



from datetime import datetime
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from pyearth.system.define_global_variables import *
from pyearth.visual.color.create_diverge_rgb_color_hex import create_diverge_rgb_color_hex


def plot_xy_data(aX_all,
                 aY_all,
                 sFilename_out,
                 iDPI_in=None,
                 aFlag_trend_in=None,
                 iReverse_y_in=None,
                 iSize_x_in=None,
                 iSize_y_in=None,
                 ncolumn_in=None,
                 dMax_x_in=None,
                 dMin_x_in=None,
                 dMax_y_in=None,
                 dMin_y_in=None,
                 dSpace_y_in=None,
                 aMarker_in=None,
                 aColor_in=None,
                 aLinestyle_in=None,
                 aLabel_point_in=None,
                 aTick_label_x_in=None,
                 aLocation_legend_in=None,
                 sLabel_x_in=None,
                 sLabel_y_in=None,
                 aLabel_legend_in=None,
                 sLocation_legend_in=None,
                 sFormat_y_in=None,
                 sTitle_in=None):
    """
    Plot a X-Y data

    Args:
        aX_all ([type]): [description]
        aY_all ([type]): [description]
        sFilename_out ([type]): [description]
        iDPI_in ([type], optional): [description]. Defaults to None.
        aFlag_trend_in ([type], optional): [description]. Defaults to None.
        iReverse_y_in ([type], optional): [description]. Defaults to None.
        iSize_x_in ([type], optional): [description]. Defaults to None.
        iSize_y_in ([type], optional): [description]. Defaults to None.
        ncolumn_in ([type], optional): [description]. Defaults to None.
        dMax_x_in ([type], optional): [description]. Defaults to None.
        dMin_x_in ([type], optional): [description]. Defaults to None.
        dMax_y_in ([type], optional): [description]. Defaults to None.
        dMin_y_in ([type], optional): [description]. Defaults to None.
        dSpace_y_in ([type], optional): [description]. Defaults to None.
        aMarker_in ([type], optional): [description]. Defaults to None.
        aColor_in ([type], optional): [description]. Defaults to None.
        aLinestyle_in ([type], optional): [description]. Defaults to None.
        aLabel_point_in ([type], optional): [description]. Defaults to None.
        aTick_label_x_in ([type], optional): [description]. Defaults to None.
        aLocation_legend_in ([type], optional): [description]. Defaults to None.
        sLabel_x_in ([type], optional): [description]. Defaults to None.
        sLabel_y_in ([type], optional): [description]. Defaults to None.
        aLabel_legend_in ([type], optional): [description]. Defaults to None.
        sLocation_legend_in ([type], optional): [description]. Defaults to None.
        sFormat_y_in ([type], optional): [description]. Defaults to None.
        sTitle_in ([type], optional): [description]. Defaults to None.
    """

    # find how many data will be plotted
    nData = len(aY_all)

    if iDPI_in is not None:
        iDPI = iDPI_in
    else:
        iDPI = 300

    if aFlag_trend_in is not None:
        aFlag_trend = aFlag_trend_in
    else:
        aFlag_trend = np.full(nData, 0)

    if iReverse_y_in is not None:
        iReverse_y = iReverse_y_in
    else:
        iReverse_y = 0

    if iSize_x_in is not None:
        iSize_x = iSize_x_in
    else:
        iSize_x = 12

    if iSize_y_in is not None:
        iSize_y = iSize_y_in
    else:
        iSize_y = 9

    if sLocation_legend_in is not None:
        sLocation_legend = sLocation_legend_in
    else:
        sLocation_legend = "upper right"

    if aLocation_legend_in is not None:
        aLocation_legend = aLocation_legend_in
    else:
        aLocation_legend = (1.0, 1.0)

    if ncolumn_in is not None:
        ncolumn = ncolumn_in
    else:
        ncolumn = 1

    if sLabel_x_in is not None:
        sLabel_x = sLabel_x_in
    else:
        sLabel_x = ''

    if sLabel_y_in is not None:
        sLabel_y = sLabel_y_in
    else:
        sLabel_y = ''

    if aLabel_legend_in is not None:
        aLabel_legend = aLabel_legend_in
    else:
        aLabel_legend = np.full(nData, '')

    if sTitle_in is not None:
        sTitle = sTitle_in
    else:
        sTitle = ''

    if aMarker_in is not None:
        aMarker = aMarker_in
    else:
        aMarker = np.full(nData, '+')

    if aColor_in is not None:
        aColor = aColor_in
    else:
        if (nData >= 3):
            aColor = create_diverge_rgb_color_hex(nData)
        else:
            if nData == 2:
                aColor = ['red', 'blue']
            else:
                aColor = ['red']

    if aLinestyle_in is not None:
        aLinestyle = aLinestyle_in
    else:
        aLinestyle = np.full(nData, '-')

    if dMax_x_in is not None:
        dMax_x = dMax_x_in
    else:
        dMax_x = np.nanmax(aX_all)
    if dMin_x_in is not None:
        dMin_x = dMin_x_in
    else:
        dMin_x = np.nanmin(aX_all)

    if dMax_y_in is not None:
        dMax_y = dMax_y_in
    else:
        dMax_y = np.nanmax(aY_all) * 1.0
    if dMin_y_in is not None:
        dMin_y = dMin_y_in
    else:
        dMin_y = np.nanmin(aY_all)  # if it has negative value, change here
    if (dMax_y <= dMin_y):
        return

    if dSpace_y_in is not None:
        iFlag_space_y = 1
        dSpace_y = dSpace_y_in
    else:
        iFlag_space_y = 0
        pass

    if aLabel_point_in is not None:
        iFlag_label = 1
        aLabel_point = aLabel_point_in
        pass
    else:
        iFlag_label = 0
        pass

    if aTick_label_x_in is not None:
        iFlag_replace_xtick = 1
        xtick_labels = aTick_label_x_in
        pass
    else:
        iFlag_replace_xtick = 0
        pass

    fig = plt.figure(dpi=iDPI)
    fig.set_figwidth(iSize_x)
    fig.set_figheight(iSize_y)
    ax = fig.add_axes([0.1, 0.5, 0.8, 0.4])

    if sFormat_y_in is not None:
        iFlag_format_y = 1
        sFormat_y = sFormat_y_in
    else:
        iFlag_format_y = 0

    # start loop for each data
    for i in np.arange(1, nData+1):

        x1 = aX_all[i-1]
        y1 = aY_all[i-1]
        ax.plot(x1, y1,
                color=aColor[i-1], linestyle=aLinestyle[i-1],
                marker=aMarker[i-1],
                label=aLabel_legend[i-1])

        # calculate linear regression
        iFlag_trend = aFlag_trend[i-1]
        if iFlag_trend == 1:
            nan_index = np.where(y1 == missing_value)
            y1[nan_index] = np.nan
            good_index = np.where(~np.isnan(y1))

            x_dummy = np.array([i.timestamp() for i in x1])
            x_dummy = x_dummy[good_index]
            y_dummy = y1[good_index]
            coef = np.polyfit(x_dummy, y_dummy, 1)
            poly1d_fn = np.poly1d(coef)
            mn = np.min(x_dummy)
            mx = np.max(x_dummy)
            x2 = [mn, mx]
            y2 = poly1d_fn(x2)
            x2 = [datetime.fromtimestamp(i) for i in x2]
            ax.plot(x2, y2, color='orange', linestyle='-.',  linewidth=0.5)

    ax.axis('on')
    ax.grid(which='major', color='grey', linestyle='--', axis='y')

    # ax.set_aspect(dRatio)  #this one set the y / x ratio

    ax.tick_params(axis="x", labelsize=10)
    ax.tick_params(axis="y", labelsize=10)

    ax.set_xmargin(0.05)
    ax.set_ymargin(0.15)

    ax.set_xlabel(sLabel_x, fontsize=12)
    ax.set_ylabel(sLabel_y, fontsize=12)
    ax.set_title(sTitle, loc='center', fontsize=15)

    ax.set_xlim(dMin_x, dMax_x)

    # if dMax_y < 1000 and dMax_y > 0.001:
    #    ax.yaxis.set_major_formatter( mpl.ticker.FormatStrFormatter('%.1f'))
    # else:
    #    ax.yaxis.set_major_formatter( mpl.ticker.FormatStrFormatter('%.1e'))
    if (iFlag_format_y == 1):
        ax.yaxis.set_major_formatter(mpl.ticker.FormatStrFormatter(sFormat_y))

    if (iFlag_space_y == 0):
        ax.yaxis.set_major_locator(mpl.ticker.AutoLocator())
        ax.yaxis.set_minor_locator(mpl.ticker.AutoMinorLocator())
    else:
        ax.yaxis.set_major_locator(mpl.ticker.MultipleLocator(dSpace_y))

    if (iReverse_y == 1):
        ax.set_ylim(dMax_y, dMin_y)
    else:
        ax.set_ylim(dMin_y, dMax_y)
        pass

    if iFlag_label == 1:
        aText = []
        for i in np.arange(1, len(aLabel_point)+1, 1):

            sLabel_point = aLabel_point[i-1]
            aText.append(
                ax.text(aX_all[1][i-1], aY_all[1][i-1], sLabel_point, color='red'))

            pass

        # adjust_text(aText, \
        #    only_move={'points':'y', 'texts':'xy'}, \
        #    arrowprops=dict(arrowstyle="->", color='r', lw=0.5) )

        pass

    if iFlag_replace_xtick == 1:
        ax.set_xticks(aX_all[0])
        ax.set_xticklabels(xtick_labels, fontsize=8)
        pass

    ax.legend(bbox_to_anchor=aLocation_legend,
              loc=sLocation_legend,
              fontsize=8,
              ncol=ncolumn)

    plt.savefig(sFilename_out, bbox_inches='tight')

    plt.close('all')
    plt.clf()



import matplotlib as mpl
def log_formatter(x):
    a, b = '{:.1e}'.format(x).split('e')
    b = int(b)
    return r'${} \times 10^{{{}}}$'.format(a, b)

def float_formatter(x):
    a = '{:.1f}'.format(x)
    return a

class MathTextSciFormatter(mpl.ticker.Formatter):
    def __init__(self, fmt="%1.2e"):
        self.fmt = fmt
    def __call__(self, x, pos=None):
        s = self.fmt % x
        decimal_point = '.'
        positive_sign = '+'
        tup = s.split('e')
        significand = tup[0].rstrip(decimal_point)
        sign = tup[1][0].replace(positive_sign, '')
        exponent = tup[1][1:].lstrip('0')
        if exponent:
            exponent = '10^{%s%s}' % (sign, exponent)
        if significand and exponent:
            s =  r'%s{\times}%s' % (significand, exponent)
        else:
            s =  r'%s%s' % (significand, exponent)
        return "${}$".format(s)
    
class OOMFormatter(mpl.ticker.ScalarFormatter):
    def __init__(self, order=0, fformat="%1.1e", offset=True, mathText=True):
        self.oom = order
        self.fformat = fformat
        mpl.ticker.ScalarFormatter.__init__(
            self, useOffset=offset, useMathText=mathText)

    def _set_order_of_magnitude(self):
        self.orderOfMagnitude = self.oom

    def _set_format(self, vmin=None, vmax=None):
        self.format = self.fformat
        if self._useMathText:
            self.format = r'$\mathdefault{%s}$' % self.format




import datetime
import pyearth.toolbox.date.julian as julian
def day_in_month(iYear_in, 
                 iMonth_in,
                 iFlag_leap_year_in=None):
    """
    Calculate how many days in a month

    Args:
        iYear_in (int): The year
        iMonth_in (int): The month

    Returns:
        int: The total day in the month
    """
    
    dummy1 = datetime.datetime(iYear_in, iMonth_in, 1)
    lJulian_start = julian.to_jd(dummy1, fmt='jd')    

    if iMonth_in < 12:
        dummy1 = datetime.datetime(iYear_in, iMonth_in+1, 1)
        lJulian_end = julian.to_jd(dummy1, fmt='jd')
        
    else:
        dummy1 = datetime.datetime(iYear_in+1, 1, 1)
        lJulian_end = julian.to_jd(dummy1, fmt='jd')

    dayinmon = int (lJulian_end - lJulian_start ) 

    if iFlag_leap_year_in is not None:
        if iFlag_leap_year_in ==0:
            if iMonth_in ==2:
                dayinmon = 28
        else:
            if iMonth_in ==2:
                dayinmon = 29
            

    return dayinmon